<?php

namespace Drupal\ip_geoloc\Form;

use Drupal\Core\Database\Connection;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Datetime\DateFormatterInterface;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Session\AccountProxyInterface;
use Drupal\ip_geoloc\Services\IpGeoLocAPI;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Pending doc.
 */
class IpLookupForm extends FormBase {

  /**
   * The IP Geolocation API service.
   *
   * @var \Drupal\ip_geoloc\Services\IpGeoLocAPI
   */
  protected $ipGeolocApi;

  /**
   * The current logged user.
   *
   * @var \Drupal\Core\Session\AccountProxyInterface
   */
  protected $currentUser;

  /**
   * The date formatter service.
   *
   * @var \Drupal\Core\Datetime\DateFormatterInterface
   */
  protected $dateFormatter;

  /**
   * The database service.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $database;

  /**
   * Create the form, adds dependency injection.
   */
  public function __construct(ConfigFactoryInterface $config_factory, IpGeoLocAPI $ipGeolocApi, AccountProxyInterface $currentUser, DateFormatterInterface $date_formatter, Connection $database) {
    parent::__construct($config_factory);
    $this->ipGeolocApi = $ipGeolocApi;
    $this->currentUser = $currentUser;
    $this->dateFormatter = $date_formatter;
    $this->database = $database;

  }

    /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('ip_geoloc.api'),
      $container->get('date.formatter'),
      $container->get('current_user'),
      $container->get('database')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'ip_geoloc_ip_lookup_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    // Migration comment:  Part of ip_geoloc_set_location_form definition.
    $config = $this->config('ip_geoloc.settings');

    $last_visit = $form_state->has('last_visit') ? $form_state->get('last_visit') : '';
    $formatted_address = $form_state->has('formatted_address') ? $form_state->get('formatted_address') : '';
    if (!empty($last_visit) || !empty($formatted_address)) {
      $prefix = '<div class="ip_geoloc_address_lookup">' . $formatted_address . (empty($last_visit) ? '' : "<br/>$last_visit") . '</div>';
    }
    $form['ip_address'] = [
      '#type' => 'textfield',
      '#title' => t('IP address'),
      '#default_value' => '',
      '#size' => 16,
      '#required' => FALSE,
      '#prefix' => isset($prefix) ? $prefix : NULL,
    ];
    if ($this->currentUser->hasPermission('administer site configuration') && $config->get('ip_geoloc_store_addresses') ? $config->get('ip_geoloc_store_addresses') : TRUE) {
      $form['store'] = [
        '#type' => 'checkbox',
        '#title' => t('If found, store on IP geolocation database for future reference.'),
        '#default_value' => FALSE,
      ];
    }
    $form['submit'] = [
      '#type' => 'submit',
      '#value' => t('Lookup'),
      '#submit' => ['ip_geoloc_ip_lookup_submit'],
    ];
    return $form;

  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    // Migration comment:  Part of _ip_geoloc_process_go_to_submit for submiting ip_geoloc_set_location_form
    // $form_state->set(['storage', 'value'], 'something');.
    $store = !empty($form_state->getStorage());

    $location = $this->ipGeolocApi->getLocationByIp(trim($form_state->get('ip_address'), TRUE, $store));


    // @TODO Acesslog table doesn't exist for d8
    if ($this->database->schema()->tableExists('accesslog')) {

      $last_visit = $this->database->query('SELECT MAX(timestamp) FROM {accesslog} WHERE hostname = :ip_address', [':ip_address' => $location['ip_address']])->fetchField();
      if ($last_visit) {
        $last_visit = $this->dateFormatter->format($last_visit, 'short');

        $form_state->set(['storage', 'value'], 'last_visit');
        $form_state->set(
          ['storage',
            $this->t('<strong>Last visit:</strong> %date',
            ['%date' => $last_visit])
          ],
          'last_visit');
      }
    }
    $formatted_address = isset($location['formatted_address']) ? t('%ip: <strong>%address</strong>', ['%ip' => $location['ip_address'], '%address' => $location['formatted_address']]) : '';

    $form_state->set(['storage', $formatted_address], 'formatted_address');

    // To preserve entered values and storage array.
    $form_state->setRebuild();
  }

}
