<?php

namespace Drupal\ip_geoloc\Plugin\Block;

use Drupal\Core\Block\BlockBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * IPGV&M: Set my location .
 *
 * @Block(
 *   id = "recent_visitors_map",
 *   admin_label = @Translation("IPGV&M: Map (Google) most recent visitors"),
 *   category = @Translation("IP Geoloc"),
 * )
 */
class RecentVisistorsMapBlock extends BlockBase {

  /**
   * {@inheritdoc}
   */
  public function build() {
    // Calling the set location form in block
    // TODO find a way to add subject.
    $block['subject'] = $this->t('Recent visitors');

    $config = $this->getConfiguration();

    $how_many = $config['ip_geoloc_recent_visitors_map_number_of_visitors'] ? $config['ip_geoloc_recent_visitors_map_number_of_visitors'] : 20;

    $block = [];

    if ($how_many > 0) {
      $map_style = $config['ip_geoloc_recent_visitors_map_div_style'] ? $config['ip_geoloc_recent_visitors_map_div_style'] : IP_GEOLOC_MAP_DIV_DEFAULT_STYLE;
      $locations = ip_geoloc_get_recent_visitor_locations($how_many);
      $map_options = $config['ip_geoloc_recent_visitors_map_options'] ? $config['ip_geoloc_recent_visitors_map_options'] : IP_GEOLOC_RECENT_VISITORS_MAP_OPTIONS;
      // TODO check if it is correct.
      $block = [
        '#theme' => 'ip_geoloc_visitor_map',
        '#locations' => $locations,
        '#div_id' => 'ip-geoloc-block-recent-visitors-map',
        '#map_options' => $map_options,
        '#map_style' => $map_style,
      ];

    }

    return $block;
  }

  /**
   * {@inheritdoc}
   */
  public function getCacheMaxAge() {
    return 0;
  }

  /**
   * {@inheritdoc}
   */
  public function blockForm($form, FormStateInterface $form_state) {
    $form = parent::blockForm($form, $form_state);

    // Retrieve existing configuration for this block.
    $config = $this->getConfiguration();

    // Add fields for block configuration.
    $form['ip_geoloc_recent_visitors_map_number_of_visitors'] = [
      '#type' => 'textfield',
      '#title' => t('Number of recent visitor locations to show on the map.'),
      '#default_value' => $config['ip_geoloc_recent_visitors_map_number_of_visitors'] ? $config['ip_geoloc_recent_visitors_map_number_of_visitors'] : 20,
      '#description' => t('Must be greater than 0. Clustering is not supported, all markers are shown individually.'),
    ];

    $form['ip_geoloc_recent_visitors_map_div_style'] = [
      '#type' => 'textfield',
      '#title' => t('CSS style attribute(s) applied to the HTML DIV element that is placeholding the map'),
      '#default_value' => $config['ip_geoloc_recent_visitors_map_div_style'] ? $config['ip_geoloc_recent_visitors_map_div_style'] : IP_GEOLOC_MAP_DIV_DEFAULT_STYLE,
      '#description' => t('The default %default_style results in a map of 300 pixels high, with a width bounded by the element that contains it. Do not enter quotes or equal signs.',
        ['%default_style' => IP_GEOLOC_MAP_DIV_DEFAULT_STYLE]),
    ];

    $form['ip_geoloc_recent_visitors_map_options'] = [
      '#type' => 'textfield',
      '#size' => 120,
      '#title' => t('Map options'),
      '#default_value' => $config['ip_geoloc_recent_visitors_map_options'] ? $config['ip_geoloc_recent_visitors_map_options'] : IP_GEOLOC_RECENT_VISITORS_MAP_OPTIONS,
      '#description' => t("The default %default_options produces a world map zoomed in to level 2. A list of map options can be found <a href='!google_map_docs'>here</a>. Remember to separate options with comma's, not semi-colons, and make sure your quotes match.",
          [
            '%default_options' => IP_GEOLOC_RECENT_VISITORS_MAP_OPTIONS,
            '!google_map_docs' => IP_GEOLOC_DOC_GOOGLE_MAP_OPTIONS,
          ]),
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function blockSubmit($form, FormStateInterface $form_state) {
    // Migration comment:  Part of ip_geoloc_block_save for recent_visitors_map delta
    // Save options common to all maps.
    $this->configuration['ip_geoloc_recent_visitors_map_div_style'] = $form_state->get('ip_geoloc_recent_visitors_map_div_style');

    // TODO Review function.
    ip_geoloc_save_map_options('ip_geoloc_recent_visitors_map_options', $form_state->get('ip_geoloc_recent_visitors_map_options'));

    $this->configuration['ip_geoloc_recent_visitors_map_number_of_visitors'] = $form_state->get('ip_geoloc_recent_visitors_map_number_of_visitors');

  }

}
